'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.default = installCmd;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _EmptyObservable = require('rxjs/observable/EmptyObservable');

var _concat = require('rxjs/operator/concat');

var _publishReplay = require('rxjs/operator/publishReplay');

var _skip = require('rxjs/operator/skip');

var _map = require('rxjs/operator/map');

var _merge = require('rxjs/operator/merge');

var _ignoreElements = require('rxjs/operator/ignoreElements');

var _install = require('./install');

var _cache = require('./cache');

var _pkg_json = require('./pkg_json');

var _build = require('./build');

var _config = require('./config');

var config = _interopRequireWildcard(_config);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * run the installation command.
 * @param  {String} cwd - current working directory (absolute path).
 * @param  {Object} argv - parsed command line arguments.
 * @return {Observable} - an observable sequence that will be completed once
 * the installation is complete.
 */
function installCmd(cwd, argv) {
	var _context;

	var isExplicit = argv._.length - 1;
	var updatedPkgJSONs = isExplicit ? (0, _pkg_json.fromArgv)(cwd, argv) : (0, _pkg_json.fromFs)(cwd);

	var nodeModules = _path2.default.join(cwd, 'node_modules');
	var target = _path2.default.relative(config.storageDir, '.');

	var resolvedAll = (_context = (_context = (_context = _map.map.call(updatedPkgJSONs, function (pkgJson) {
		return { pkgJson: pkgJson, target: target, isEntry: true };
	}), _install.resolveAll).call(_context, nodeModules, undefined, isExplicit), _skip.skip).call(_context, 1), _publishReplay.publishReplay).call(_context).refCount();

	var initialized = (_context = (0, _cache.init)(), _ignoreElements.ignoreElements).call(_context);
	var linkedAll = _install.linkAll.call(resolvedAll, nodeModules);
	var fetchedAll = _install.fetchAll.call(resolvedAll, nodeModules);
	var installedAll = (0, _merge.mergeStatic)(linkedAll, fetchedAll);

	var builtAll = argv.build ? _build.buildAll.call(resolvedAll, nodeModules) : _EmptyObservable.EmptyObservable.create();

	var saved = argv.save || argv['save-dev'] || argv['save-optional'] ? _pkg_json.save.call(updatedPkgJSONs, cwd) : _EmptyObservable.EmptyObservable.create();

	return (0, _concat.concatStatic)(initialized, installedAll, saved, builtAll);
}